/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Annozilla.
 *
 * The Initial Developer of the Original Code is
 * Matthew Wilson <matthew@mjwilson.demon.co.uk>.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Eliot Setzer
 *   C. Greg Hagerty <cgreg@cgreg.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const ANNOZILLA_XUL_NS = "http://annozilla.mozdev.org/xul/2005";

// The current design of Annozilla is intended to allow multiple Annozilla
// objects simultaneously (such as one for each tab in a tabbed browser), but
// this is not used or tested yet.
var gAnnozilla; // was set by initialize.js, now by initAnnozilla via overlay
var gAnnozillaBrowser; // for tab handling

function initAnnozilla () {
    gAnnozilla = new Annozilla();
    window.addEventListener("unload", 
                            function (e) {gAnnozilla.unload();},
                            false);

    gAnnozillaBrowser = window.top.gBrowser;
    var container = gAnnozillaBrowser.tabContainer;
    container.addEventListener("TabSelect", gAnnozilla.tabSelected, false);
    container.addEventListener("TabClose", exampleTabRemoved, false);
}


function Annozilla() {
    dump("Annozilla is here\n");
    // test for required components
    if (! Annozilla._hasRequiredComponents()) {
        return;
    }

    this.windowIndex = 0;

    this.bundle = srGetStrBundle("chrome://annozilla/locale/annozilla.properties");

    // initialize annotations service
    this.annotService = Components.classes[Annozilla.ANNOTATIONS_SERVICE_CONTRACTID].getService().QueryInterface(Components.interfaces.nsIAnnotationsService);

    // get the annozilla AnnotationsApplication object
    this.app = this.annotService.registerApplication("annozilla");

    // add our observer
    this.app.addObserver(this);

    // icons module
    this.icons = new IconManager(this);

    // sidebar module, when loaded
    this.sidebar = null;

    // prefs module
    this.prefs = new AnnozillaPrefs(this);

    // This is necessary to prevent _addAnnotationToContainer from failing when
    // an annotation is created before any annotations have ever been loaded by
    // the sidebar.
    this.annotations = {};

    this.defaultBodyDestination = window.document.webNavigation;

}

Annozilla.UNKNOWN_REASON_URL = "about:blank";

Annozilla.ANNOTATIONS_SERVICE_CONTRACTID = "@mozilla.org/annotations_service;1";
Annozilla.XPOINTER_SERVICE_CONTRACTID = "@mozilla.org/xpointer-service;1";

Annozilla.PAGE_LOADING = 0;
Annozilla.PAGE_COMPLETED = 1;
Annozilla.PAGE_FAILED = 2;

Annozilla.ANNOT_NS = "http://www.w3.org/2000/10/annotation-ns#";
Annozilla.RDF_NS   = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
Annozilla.DC10_NS    = "http://purl.org/dc/elements/1.0/";
Annozilla.DC11_NS    = "http://purl.org/dc/elements/1.1/";
Annozilla.THREAD_NS= "http://www.w3.org/2001/03/thread#";
Annozilla.XHTML_NS = "http://www.w3.org/1999/xhtml";
Annozilla.TYPE_NS = "http://www.w3.org/2000/10/annotationType#";

Annozilla.prototype = {
    windowIndex: undefined,
    annotations: undefined,
    bundle: undefined,
    userLoading: undefined,
    uriState: undefined,
    serversResponded: undefined,
    serverCount: undefined,
    defaultBodyDestination: undefined
};

// The previous value for this ("annotation-body") produced warnings (about the
// '-' not being a legal character) in a version of Firefox compiled with
// debugging messages.
Annozilla.prototype.WINDOW_ID_PREFIX = "annotationBody";

Annozilla.prototype.unload = function() {
    // remove observer
    this.app.removeObserver(this);

    // tell prefs we're shutting down
    this.prefs.destructor();
}
    

Annozilla.prototype.QueryInterface = function(iid) {
    if (! (iid.equals(Components.interfaces.nsIAnnotationsObserver) ||
           iid.equals(Components.interfaces.nsISupports) ) ) {
               throw Components.results.NS_ERROR_NO_INTERFACE;
           }
    return this;
}

Annozilla.prototype.insertIconsPrefChanging = function(insert){
    if(this.icons.insertIconsPrefChanging){
        this.icons.insertIconsPrefChanging(insert);
    }
}

// Return the range specified by the fragment.
Annozilla.prototype.resolveFragment = function(fragment){
    return xptrResolveFragment(fragment, window._content.document);
}

// Return the range to which the annotation applies.
Annozilla.prototype.resolveAnnotation = function(annotation){
    return xptrResolveAnnotation(annotation, window._content.document);
}

/**
 * Determine whether the Mozilla Composer HTML editor is available.
 */
Annozilla.prototype.isMozillaEditorAvailable = function () {
/* Prefered test is to check whether the htmleditor class is installed;
   however even on MozillaFirebird 1.0 this test returns true.
   Backup test is to test for the existence of the element "key_editPage"
   in the top browser window - this is associated with the "File ->
   Edit Page" menu option. Hopefully this hackiness will be removed
   at some point in the future.
 */
    return (("@mozilla.org/editor/htmleditor;1" in Components.classes) &&
            window.top.document.getElementById ("key_editPage") != undefined);
}


/*
 * Currently, annotation collection is cleared and reloaded for each tab switch
 * But addAnnotations/_addNext checks each annotation URI is for current page
 *  so that if we want to avoid reload we could accumulate annotations
 *  for open pages by not resetting the collection
 */
Annozilla.prototype.tabSelected = function (event) {
    var browser = gAnnozillaBrowser.selectedTab;
    // browser is the newly selected tab
    var url = content.document.location.toString();
    dump("Selected tab for "+url+"\n");
    this.currentURI = url;
    
    if (!gAnnozilla.autoLoadAllowSite(url)) { 
        //clear sidebar?
        return; 
    }
    
    var total = 0;
    for (uri in gAnnozilla.annotations) { total++; }
    dump(" Total of "+total+" annotations are currently loaded\n");
    
    // Just reload everything for now, though icons will be unnecessarily 
    // removed (they would be "found" by IconManager otherwise)
    //  cgh: commented out removeIcons to fix this! 
    if (gAnnozilla.sidebar) {
        //kludgy: how about having sidebar observe Annozilla?
        gAnnozilla.sidebar.userLoadOnCommand();
    } else {
        gAnnozilla.userLoadOnCommand();
    }
}

function exampleTabRemoved(event)
{
  var browser = event.target.linkedBrowser;
  // browser is the XUL element of the browser that's been removed
  dump("Close tab for "+content.document.location+"\n");
}



/*
function exampleTabAdded(event)
{
  var browser = event.target.linkedBrowser;
  // browser is the XUL element of the browser that's been added
}

function exampleTabMoved(event)
{
  var browser = event.target.linkedBrowser;
  // browser is the XUL element of the browser that's been moved
}

// During initialisation
var container = gAnnozillaBrowser.tabContainer;
container.addEventListener("TabOpen", exampleTabAdded, false);
container.addEventListener("TabMove", exampleTabMoved, false);
container.addEventListener("TabClose", exampleTabRemoved, false);
container.addEventListener("TabSelect", exampleTabSelected, false);

// When no longer needed
container.removeEventListener("TabOpen", exampleTabAdded, false);
container.removeEventListener("TabMove", exampleTabMoved, false);
container.removeEventListener("TabClose", exampleTabRemoved, false);
container.removeEventListener("TabSelect", exampleTabSelected, false);
*/


/**
 * Return the chrome URL used for editing annotation bodies.
 */
Annozilla.prototype.getEditorUrl = function () {
    return this.isMozillaEditorAvailable() ?
           "chrome://annozilla/content/editor/composer.xul" :
           "chrome://annozilla/content/editor/editor.xul";

}

Annozilla.prototype.removeIcons = function(){ this.icons.removeIcons(); };

/**
 * Responds to a user pressing the load button
 * by loading annotations for the current page.
 * Signifies that from here on out, we should
 * pay attention to load callbacks instead of autoLoad
 * callbacks.
 */   
Annozilla.prototype.userLoadOnCommand = function() {
    this.userLoading = true;

    //any existing icons will be "found" by IconManager
    // we may have been called from tabSelect
    //this.removeIcons();

    this.annotations = {};
    this.serversResponded = 0;
    this.serverCount = this.currentServerCount();

    if (this.serverCount == 0) {
        return;
    }

    // do the load
    this.app.loadAnnotations(this.currentURI());
}

Annozilla.prototype.openEditorWindow = function(
    windowId,
    windowOptions,
    annotation,
    baseURL,
    fragment,
    bodyURI
){
    var w = window.openDialog(
        this.getEditorUrl(),
        windowId,
        windowOptions,
        annotation,
        baseURL,
        fragment,
        bodyURI
    );

    if((!annotation)&&fragment) this.icons.addIconForEditorWindow(w, fragment);
    else throw(
        "openEditorWindow requires either a fragment or an existing annotation"
    );
}

Annozilla.prototype.post = function(baseURL, fragment){
    this.openEditorWindow(
        this.WINDOW_ID_PREFIX + this.windowIndex,
        "chrome,resizable,dialog=no",
        null,
        baseURL,
        fragment,
        null
    );
    this.windowIndex++;
};

Annozilla.prototype.postOnCommand = function() {

    // get the xptr
    var seln = window._content.getSelection();
    if (seln.anchorNode == null) {
        // it might be a good idea to move into html[1]/body[1]? if it exists.
    }
 
    var baseURL = this.stripURL(window._content.location.href);
    var xptr = xptrCreateXPointer(seln, window._content.document);
    this.post(baseURL, xptr);
}

Annozilla.prototype.update = function(annotationURI) {
    if (! (annotationURI && this.annotations[annotationURI]) ) {
        window.alert("Unable to delete the selected annotation due to an internal error.  Try reloading the annotations for the page.");
        return;
    }

    var annotation = this.annotations[annotationURI].annot.clone();

    var bodyURI;
    // if this is a local annotation
    // we must generate a local url for its body
    if (annotation.serverName == this.app.LOCALSERVER_URI){
        bodyURI = LocalServerUtils.createTempURIFromString(
            annotation.getBodyText()
        );
    }else{
        bodyURI = annotation.bodyURI;
    }
  
    this.openEditorWindow(
        this.WINDOW_ID_PREFIX + this.windowIndex,
        "chrome,resizable,dialog=no",
        annotation,
        null,
        null,
        bodyURI
    );
    this.windowIndex++;
}

Annozilla.prototype.updateOnCommand = function(annotationURI) {
    this.update(annotationURI);
}

Annozilla.prototype.deleteOnCommand = function(annotationURI) {
    if (! (annotationURI && this.annotations[annotationURI]) ) {
        window.alert("Unable to delete the selected annotation due to an internal error. " +
                     " Try reloading the annotations for the page.");
        return;
    }

    var annotation = this.annotations[annotationURI].annot;
    this.app.deleteAnnotation(annotation);

}


Annozilla.prototype.loadBodyOnCommand = function(annotationURI) {
  if (! (annotationURI && this.annotations[annotationURI]) ) {
    window.alert("Unable to load the selected annotation's body due to an internal error. " +
                 " Try reloading the annotations for the page.");
    return;
  }
  
  var annotation = this.annotations[annotationURI].annot;
  var bodyURI;

  // if this is a local annotation
  // we must generate a local url for its body
  if (annotation.serverName == this.app.LOCALSERVER_URI) {
      bodyURI = LocalServerUtils.createTempURIFromString(annotation.getBodyText());
  }
  else {
    bodyURI = annotation.bodyURI;
  }
  
  if (this.prefs.getPref("loadInNewWindow")) {
    window.open(bodyURI, "_blank","width=600,height=320,scrollbars=yes,resizable=yes","");
  }
  else {
    this.loadAnnotationBody(bodyURI);
  }
}

Annozilla.prototype.scrollIntoViewOnCommand = function() {
  var  annotationURI = this.sidebar.getSelectedAnnotationURI();
  if (! (annotationURI && this.annotations[annotationURI]) ) {
    window.alert("Unable to load the selected annotation's body due to an internal error. " +
                 " Try reloading the annotations for the page.");
    return;
  }
  var annotation = this.annotations[annotationURI].annot;
  var range = xptrResolveAnnotation(annotation, window._content.document);
  if (range) {
      var startNode = range.startContainer;
      while (startNode.nodeType!=Node.ELEMENT_NODE) 
          startNode = startNode.parentNode;
     startNode.scrollIntoView();
  }
}

// TODO should this be in the service?
// well, it is, but we still need to check sometimes
// check for never as well
Annozilla.prototype.autoLoadAllowSite = function(url){
    if (this.prefs.getPref("autoLoadMulti") == "never") {
        return false;
    }
    if (this.prefs.getPref("autoLoadMulti") == "sites") {
        var allowSite = false;
        var sites = this.prefs.getPref("autoLoadSites").split(" ");
        for (var ix=0; ix<sites.length; ix++) {
            if (url.indexOf(sites[ix]) == 0) {
                allowSite = true;
                dump("Autoload allowed for "+url+"\n");
                break;
            }
        }
        if (!allowSite) {
            return false;
        }
    }
    return true;
}

/*
Annozilla.prototype.resolveAnnotations = function(){
    // Resolve all outstanding annotations.
    for(uri in this.annotations) {
        var annotationObj = this.annotations[uri];

        if (! annotationObj.annot.data) {
            this._debugBadAnnotation(annotationObj.annot);
            continue;
        }

        annotationObj.range = this.resolveAnnotation(
            annotationObj.annot,
            window._content.document
        );
    }
}
*/

/**
 * Called when a page begins loading in the 
 * browser if the application has 
 * autoLoad turned on.  
 *
 * Note: only called if autoLoad is enabled.
 *
 * @param url string of url loading
 * @return whether or not we care to load
 */
Annozilla.prototype.autoLoadOnBeginPageLoad = function(url, win) {

    // ensure correct window
    if (win != window.content) {
            //return; // commented out because it can abandon legitimate autoload operations
    }


    // TODO should this be in the service? 
    //CGH: Yes, it is, but calls here either way to facilitate clear
    //if(!this.autoLoadAllowSite(url)) return;

    this.uriState = Annozilla.PAGE_LOADING;
    this.userLoading = false;
    this.annotations = {};

    this.serversResponded = 0;
    this.serverCount = this.currentServerCount();

}

/**
 * Called when a page successfully finishes loading.
 *
 * Note: only called if autoLoad is enabled.
 *
 * @param url string of url finished loading
 * @param this.annotations nsISimpleEnumeration of annotations for url
 */
Annozilla.prototype.autoLoadOnSuccessPageLoad = function(url, win) {

    // ensure correct window
    if (win != window.content) {
        return;
    }

    this.uriState = Annozilla.PAGE_COMPLETED;

    // we still gets notified here on autoload rejection, so check again
    if (!this.autoLoadAllowSite(url)) { 
        return; 
    }

    this.addAnnotations(this.annotations);
}

/**
 * Called when a page that was loading is
 * unable to load (or is cancelled by the user).
 *
 * Note: only called if autoLoad is enabled.
 *
 * @param url string of url aborted
 * @param message string error message
 */
Annozilla.prototype.autoLoadOnFailPageLoad = function(url, win) {

    // ensure correct window
    if (win != window.content) {
        return;
    }

    this.uriState = Annozilla.PAGE_FAILED;
}


/**
 * Success callback function to return the results for an
 * automatic annotations load (autoLoad).
 * This function returns the annotations for a single server, designated by serverURI.  
 * It will be called once for each server that successfully responds.
 *
 * @param serverURI string uri of the annotations server responding
 * @param annotatedURI string uri that the annotations annotate
 * @param annotations nsISimpleEnumerator of nsIAnnotations for annotatedURI that
 *                    were on serverURI annotations server
 */
Annozilla.prototype.autoLoadAnnotationsForServer = function(serverURI, win, annotatedURI, annotationsLength, annotations){

    // ensure correct window
    if (win != window.content) {
        return;
    }

    // if there is a user load in progress, ignore these annotations
    if (this.userLoadInProgress()) {
        dump("ignoring autoLoad annotations from " + serverURI + " about " + annotatedURI + " since userLoad in progress\n");
        return;
    }

    this.serversResponded += 1;

    this._processAnnotationsWithAutoLoadInfo(
        this.currentURIState(),
        annotations
    );
}

/**
 * Callback function to indicate failure for an automatic
 * annotations load (autoLoad).
 * This function will be called once for each annotations server that
 * fails in the autoloading process. 
 *
 * @param serverURI string uri of the annotations server that failed
 * @param annotatedURI string uri for which annotations were requested
 * @param message string error message
 */ 
Annozilla.prototype.autoLoadFailureForServer = function(serverURI, win, annotatedURI, message) {

    // ensure correct window
    if (win != window.content) {
        return;
    }

  this.serversResponded += 1;
}

/**
 * Success callback function to return the results for a loadAnnotations request
 *  - OR - an automatic autoLoad request.  This function returns the 
 * annotations for one server, designated by serverURI.  
 *
 * @param serverURI string uri of the annotations server responding
 * @param annotatedURI string uri that the annotations annotate
 * @param annotations nsISimpleEnumerator of nsIAnnotations for annotatedURI
 *                    that were on serverURI annotations server
 */
Annozilla.prototype.loadAnnotationsOnSuccess = function(serverURI, annotatedURI,  annotationsLength, annotations){
    // If we're autoLoading, ignore these annotations.
    if (!this.userLoading) {
        dump(
             "ignoring loadAnnotationsOnSuccess of " + annotatedURI + " from "
             + serverURI + " because userLoad not in progress\n"
             );
        return;
    }
    
    // If we've moved on to a new page, ignore these annotations.
    if (annotatedURI != this.currentURI()) {
        dump(
             "ignoring loadAnnotationsOnSuccess of " + annotatedURI + " from "
             + serverURI + " because it pertains to old URI.  currentURI(): "
             + this.currentURI() + "\n"
             );
        return;
    }
    
    try {
        this.serversResponded += 1;

        this._processAnnotationsWithAutoLoadInfo(Annozilla.PAGE_COMPLETE,
                                                 annotations);

        /* Just go for it ^ assuming page is complete
        // If autoLoad is enabled, we can benefit from its alerts about when
        // the page is fully loaded.
        if (this.isAutoLoadEnabled()) {
            this._processAnnotationsWithAutoLoadInfo(
                 this.uriState,  
                 annotations);
        } else {
            // Otherwise, we have to wing it. for now we'll assume that the page
            // is loaded, since there's no way that I know to check whether the
            // DOM is fully constructed.
            
            for(var i = 0; i < annotations.length; i++) {
                var current = annotations[i];
                
                // testing purposes
                if (! current.data) {
                    this._debugBadAnnotation(current);
                    continue;
                }
                
                var range = this.resolveAnnotation(
                    current,
                    window._content.document
                    );
                // Add to the overall collection.
                this._addAnnotationToContainer(
                    current,
                    range,
                    this.annotations
                    );
            }
        }
        */
        
    } catch (e) {
        var consoleService=Components.classes['@mozilla.org/consoleservice;1'];
        var console = consoleService.getService(
                         Components.interfaces.nsIConsoleService
                         );
        console.logStringMessage("Annozilla: error loading annotations");
        Components.utils.reportError (e);
    }
}

/**
 * Callback function to indicate failure for a loadAnnotations request
 *  - OR - an automatic autoLoad request.  This function indicates failure
 * for one particular annotations server, designated by serverURI.
 *
 * @param serverURI string uri of the annotations server responding
 * @param annotatedURI string uri for which annotations were requested
 * @param message string error message
 */
Annozilla.prototype.loadAnnotationsOnFailure = function(serverURI, annotatedURI, message) {
    dump("Annozilla: loadAnnotationsOnFailure for server: " + serverURI + " w/ no annotations of " + annotatedURI +
         " because of error: " + message + "\n");
}

/**
 * Called when an annotation post request succeeds.
 *
 * @param newAnnotation nsIAnnotation the newly created annotation
 */
Annozilla.prototype.postAnnotationOnSuccess = function(newAnnotation) {
    // if this annotation applies to a page we're no longer viewing
    var uri = newAnnotation.data.getProperty(Annozilla.ANNOT_NS, "annotates");
    if (uri != this.currentURI()) {
        dump("posted annotation applies to page no longer being viewed\n");
        dump(" ('"+uri+"' while viewing '"+this.currentURI()+"')\n");
        return;
    }

    var range = this.resolveAnnotation(newAnnotation, window._content.document);

    // add to the overall collection
    this._addAnnotationToContainer(
        newAnnotation,
        range,
        this.annotations
    );

    // calling IconManager here avoids additional resolve
    this.icons.addAnnotation(newAnnotation, range);

    //ditto sidebar
    if (this.sidebar) {
        this.sidebar.addAnnotation(newAnnotation, range);
    }
}

Annozilla.prototype.byeByeSidebar = function () {
    dump("Annozilla: Bye bye Sidebar\n");
    this.sidebar = null;
}

/**
 * Called when an annotation post fails.
 *
 * @param message string error message
 */
Annozilla.prototype.postAnnotationOnFailure = function(message) {}



/**
 * Called when an annotation update succeeds.
 *
 * @param updatedAnnotation nsIAnnotation that has been updated on server
 */
Annozilla.prototype.updateAnnotationOnSuccess = function(updatedAnnotation) {
    var uri = updatedAnnotation.annotationURI;
    if (this.annotations && this.annotations[uri]) {
        var oldRange = this.annotations[uri].range;
        // update the old annotation
        this.annotations[uri] = {
            annot: updatedAnnotation,
            range: oldRange
        };
    }
}

/**
 * Called when an annotation update fails.
 *
 * @param url string url of the annotation that could not be updated
 * @param message string error message
 */
Annozilla.prototype.updateAnnotationOnFailure = function(url, message) {}



/**
 * Called when an annotation is successfully deleted.
 *
 * @param url string url of deleted annotation
 */
Annozilla.prototype.deleteAnnotationOnSuccess = function(url) {
    if (this.annotations && this.annotations[url]) {
        delete this.annotations[url];
    }
}

/**
 * Called when an annotation delete fails.
 *
 * @param url string url of the annotation that could not be deleted
 * @param message string error message
 */
Annozilla.prototype.deleteAnnotationOnFailure = function(url, message) {
    window.alert("Unable to delete annotation " + url + "\nThe server responded: " + message + "\n");
}

/**
 * Called when the replies to an annotation have been successfully
 * loaded.
 * 
 * @param annotation nsIAnnotation for which replies were requested
 */
Annozilla.prototype.getRepliesOnSuccess = function(annotation, repliesCount, replies) {}

/**
 * Called when getting the replies to an annotation fails.
 *
 * @param annotation nsIAnnotation for which replies were requested
 * @param message string error message
 */
Annozilla.prototype.getRepliesOnFailure = function(annotation, message) {}

/**
 * Called an annotation has successfully been replied to.
 * 
 * @param original nsIAnnotation that existed originally
 * @param reply nsIAnnotation reply created
 */
Annozilla.prototype.replyToAnnotationOnSuccess = function(original, reply) {}

/**
 * Called when replying to an annotation fails.
 *
 * @param original nsIAnnotation to which we attempted to reply
 * @param message string error message
 */
Annozilla.prototype.replyToAnnotationOnFailure = function(original, message) {}

/**
 * Called after a loadAnnotationBody request
 * when the body text of the annotation has been successfully loaded.
 *
 * @param annotation nsIAnnotation with updated body text
 * @param bodyText string text of the annotation's body
 */
Annozilla.prototype.loadAnnotationBodyOnSuccess = function(annotation, bodyText) {}

/**
 * Called after a loadAnnotationBody request when
 * the body text fails to load.
 *
 * @param annotation nsIAnnotation that body was requested for (unmodified)
 * @param message string error message
 */
Annozilla.prototype.loadAnnotationBodyOnFailure = function(annotation, message) {}

Annozilla.prototype.setDefaultBodyDestination = function(destination){
    this.defaultBodyDestination = destination;
}

Annozilla.prototype.loadAnnotationBody = function(bodyURI, destination){
    if(destination) destination.loadURI(bodyURI);
    else this.defaultBodyDestination.loadURI(bodyURI);
}

Annozilla._hasRequiredComponents = function() {
    if (! Components.classes[Annozilla.ANNOTATIONS_SERVICE_CONTRACTID]) {
        window.alert("Annozilla requires the Annotations Service.  Please install it " +
                     "from http://annozilla.mozdev.org/installation/");
        return false;
    }
    if (! Components.classes[Annozilla.XPOINTER_SERVICE_CONTRACTID]) {
        window.alert("Annozilla requires the XPointerService from XPointerLib. " +
                     "Please install it from http://xpointerlib.mozdev.org/installation.html");
        return false;
    }
    
    return true;
}

/**
 * Private function to strip query strings off of URLs. (we don't want to be
 * nosy!)
 * Also strips off fragment IDs.
 *
 * @param url String URL to strip query strings from
 * @return URL string stripped of queries
 */
Annozilla.prototype.stripURL = function (url) {
    if(!url || ('string'!= typeof(url)) || ('' == url) || url == 'about:blank'){
        return 'about:blank';
    }

    // Strip off any query strings (Don't want to be too nosy).
    var queryPart = url.indexOf('?');
    if (queryPart != -1) {
	url = url.slice(0, queryPart);
    }

    url = url.replace (/#.*/,'');

    return url;    
}

Annozilla.prototype.currentURI = function(){
    return this.stripURL(window._content.location.href);
};

Annozilla.prototype.currentURIState = function(){ return this.uriState; };

Annozilla.prototype.userLoadInProgress = function(){ return this.userLoading; };

Annozilla.prototype.isAutoLoadEnabled = function() {
    return (this.prefs.getPref("autoLoadMulti") != "never");
};

Annozilla.prototype.currentServerCount = function(){
    return this.app.getServerCount();
};

Annozilla.prototype._addAnnotationToContainer = function (aAnnotation, aRange, aContainer) {
    aContainer[aAnnotation.annotationURI] = {
        annot: aAnnotation,
        range: aRange
    };
};

Annozilla.prototype._debugBadAnnotation = function(annotation) {
    dump("annotation.data is NULL.  enumerating properties:\n");
    dump("serverName: " + annotation.serverName + "\n");
    dump("annotationURI: " + annotation.annotationURI + "\n");
    dump("bodyURI: " + annotation.bodyURI + "\n");
    dump("getBodyText: " + annotation.getBodyText() + "\n");
    dump("setting bodyText.\n");
    annotation.setBodyText("New test body text");
    dump("new body text: " + annotation.getBodyText() + "\n");
}

Annozilla.prototype._processAnnotationsWithAutoLoadInfo = function(state, annotations) {
    if (state == Annozilla.PAGE_LOADING) {
        // add the annotations without resolving their context
        for (var i = 0; i < annotations.length; i++) {
            this._addAnnotationToContainer(annotations[i], null, 
                                           this.annotations);
        }
    }
    else { //if (state == Annozilla.PAGE_COMPLETED) { or otherwise
        var currentAnnotationsBatch = {};
        
        for (var i = 0; i < annotations.length; i++) {
            var current = annotations[i];
            
            if (! current.data) {
                this._debugBadAnnotation(current);
                continue;
            }
            
            // Defer resolution and sidebar until it can be done in succession
            var range = null;
            
            // add to the current batch
            this._addAnnotationToContainer(current, range, 
                                           currentAnnotationsBatch);
            
            // add to the overall collection
            this._addAnnotationToContainer(current, range, 
                                           this.annotations);
        }
        this.addAnnotations(currentAnnotationsBatch);
    }
}


/**
 * Add annotations (asynchronously or not)
 *
 * @param annotations Collection of annotations to add
 */
var gAsynchLoading = false; // Semaphore allows user to stop load by setting
var gAList; // List of annotations to be anynchronously loaded

Annozilla.prototype.isLoadingAnnotations = function () {
    return gAsynchLoading;
}

Annozilla.prototype.addAnnotations = function (annotations) {
    gAsynchLoading = true;
    //var menuitem = window.document.getElementById('stop-load');
    //menuitem.hidden = 0;
    //this.operationInProgress("LOADING");
    gAList = [];
    for (uri in annotations) {
        gAList.push(annotations[uri].annot);
    }
    dump("Adding "+gAList.length+" annotations asynchronously\n");
    setTimeout(_addNext, 0, 0, window._content.document, this);
}

Annozilla.prototype.userStopLoadCommand = function () {
    gAsynchLoading = false;
    gAList = [];
    //TODO: SIDEBAR
    //var menuitem = window.document.getElementById('stop-load');
    //menuitem.hidden = 1;
    //this.operationComplete("LOAD_ERROR"); //define LOAD_ABORTED?
    window._content.status = "Annotation loading stopped.";
}


// setTimeout doesn't seem to like object methods?
var _addNext = function (i, doc, that) {
    if (gAsynchLoading && (i < gAList.length)) {
        try {
            dump(i+"!");
            var annot = gAList[i]; //consider popping this?
            var context = annot.data.getProperty(ANNOT_NS,"context");
            var shortURI = doc.documentURI.lastIndexOf("/")+1;
            shortURI = doc.documentURI.substr(shortURI); 
            if (context.indexOf(shortURI) == -1) {
                //SKIP ANNOTATIONS STORED FOR OTHER PAGES
                dump("(skip)");
            } else {
                var range = xptrResolveAnnotation(annot, doc);
                //var URI = annot.annotationURI;
                //that.annotations[URI].range = range;
                // this actually blew away range.toString?
                // maybe we don't need to save it anyway
                
                if (range) {
                    that.icons.addAnnotation(annot, range);
                    if (that.sidebar != null) {
                        that.sidebar.addAnnotation(annot, range);
                    }
                }
            }
            setTimeout(_addNext, 0, i+1, doc, that);
            
        } catch (e) {
            console.logStringMessage("Annozilla: error loading annotations");
            Components.utils.reportError (e);
            //TODO: SIDEBAR
            //that.operationComplete("PAGE_LOAD_ERROR");
        }
    } else {
        //TODO: SIDEBAR
        //that.operationComplete("LOAD_OK");
        //var menuitem = window.document.getElementById('stop-load');
        //menuitem.hidden = 1;
        gAList = [];
        gAsynchLoading = false;
        window._content.status = "Annotations loaded.";
        dump("\nDone!\n");
    }
}

