/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Annozilla.
 *
 * The Initial Developer of the Original Code is
 * Matthew Wilson <matthew@mjwilson.demon.co.uk>.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function closeWindow(){
    setTimeout("window.close()",500);
}

/**
 * @param annozilla Annozilla object
 * @param annotation if updating, an annotation.  if new post, then null.
 * @param xptr if updating, then null.  if new post, then xptr string.
 */
function CommonEditor(annotation, baseURL, xptr, bodyURI) {
    this.prefs = new AnnozillaPrefs(this);
    this.bundle = srGetStrBundle(
        "chrome://annozilla/locale/annozilla.properties"
    );
    this.editorBundle = srGetStrBundle(
        "chrome://annozilla/locale/editor.properties"
    );

    var defaultCreator  = this.prefs.getPref("creator");
    var defaultLanguage = this.prefs.getPref("language");
    var defaultCharset;

    if(annotation){
        // defaultCharset will not be used by PostObserver.

        defaultCharset = null;
        this.updating = true;
    }else{
        // defaultCharset will be used by PostObserver.

        defaultCharset = window.QueryInterface(
            Components.interfaces.nsIDOMWindow
        ).QueryInterface(
            Components.interfaces.nsIInterfaceRequestor
        ).getInterface(
            Components.interfaces.nsIDocCharset
        ).charset;
        this.updating = false;
    }

    this.makeUpdate = function(){
        var t = this;
        return function(){
            if(t.po.success){
                var keepOpen = document.getElementById("keepOpen").checked;
                if(!keepOpen){
                    closeWindow();
                    return;
                }
                var messageName;
                if(t.updating){
                    messageName = "UPDATE_SUCCESS_STATUS"
                }else{
                    messageName = "POST_SUCCESS_STATUS"
                    t.updating = true;
                }

                document.getElementById("statusbar-text").setAttribute(
                    "label",
                    this.editorBundle.GetStringFromName(messageName)
                );

                document.getElementById("posting").mode = "determined";
                document.getElementById("post").disabled = false;
                document.getElementById("update").disabled = false;
                document.getElementById("cancel").disabled = false;
            }

            // set the actual XUL fields
            var doc = document;
            doc.getElementById("annotationType").value = t.po.type;
            doc.getElementById("annotationLanguage").value = t.po.language;
            doc.getElementById("annotationAuthor").value = t.po.creator;
        };
    };

    this.setElementsToDefaultValues = function(){
        var doc = document;
        var keepEditWindowOpen = this.prefs.getPref("keepEditWindowOpen");
        doc.getElementById("keepOpen").checked = keepEditWindowOpen;
        // updating
        if (this.updating) {
            doc.getElementById("post").setAttribute("style",  "display: none");
            doc.getElementById("update").setAttribute("style","display: block");
            this.setupServers(true, this.po.annotation.serverName);
        }
        // posting
        else {
            doc.getElementById("post").setAttribute("style",  "display: block");
            doc.getElementById("update").setAttribute("style","display: none");
            this.setupServers(false, this.po.server);
        }

        // setup the HTML editor
        var editURI = this.po.bodyURI ? this.po.bodyURI : 'about:blank';
        this.editor.loadUrl(editURI);

        // setup spell-check
        if (! CommonEditor.isSpellCheckerInstalled()) {
            this.hideItem("spellingButton");
        }
        else {
            this.setElementEnabled(doc.getElementById("spellingButton"), true);
        }
    };

    this.po = new PostObserver(  // This calls this.makeUpdate().
        annotation,
        baseURL,
        xptr,
        bodyURI,
        this.makeUpdate(),
        defaultCreator,
        defaultLanguage,
        defaultCharset
    );

    this.po.update();

    this.editor = getEditorWrapper();

    this.setElementsToDefaultValues();

    // set a window closing observer
    var that = this;
    window.addEventListener(
        "unload",
        function() { that.windowClosing(); },
        false
    );
}

CommonEditor.prototype = {};

CommonEditor.prototype.setKeepOpen = function() {
    var prefs = Components.classes["@mozilla.org/preferences;1"].getService(
        Components.interfaces[Components.interfaces.nsIPref]
    );
    if ( prefs ) {
        prefs.SetBoolPref(
            "annozilla.keepEditWindowOpen",
            document.getElementById("keepOpen").checked
        );
    }
};
	
CommonEditor.prototype.windowClosing = function() {
    this.po.stopObserving();
    this.prefs.destructor();
};

/**
 * @param onlyOneServerAllowed boolean whether only the defaultServer may be used
 * @param defaultServer string name of defaultServer to select, or null if none
 */
CommonEditor.prototype.setupServers = function(
    onlyOneServerAllowed,
    defaultServer
){

    // get the preferences directory
    var fileLocator = Components.classes[
        "@mozilla.org/file/directory_service;1"
    ].getService(Components.interfaces.nsIProperties);  
    var file = fileLocator.get("ProfD", Components.interfaces.nsIFile);
    
    // append servers filename
    file.append(PostObserver.SERVERS_FILENAME);	
    
    // get the data source
    var urlSpec = CommonEditor.getURLSpecFromFile(file);
    var ds = gRDF.GetDataSourceBlocking(urlSpec);
    
    // intialize the listbox
    var list = document.getElementById("serverList");
    list.database.AddDataSource(ds);
    list.setAttribute("ref", PostObserver.SERVERS_ROOT_URN);

    if (defaultServer) {
	// this is the bad thing that happens when 
	// interface designers don't put length fields
	// on lists (bad mozilla, bad)
	for(var i=0; true; i++) {
	    list.selectedIndex = i;
	    if (list.value == defaultServer) {
		break;
	    }
	}
    }
    else {
	list.selectedIndex = 0;
    }

    if (onlyOneServerAllowed) {
	list.disabled = true;
    }
};
   

CommonEditor.prototype.editorOnLoad = function(url, listener) {
  var editorShell = document.getElementById("edit-frame").editorShell;
  editorShell.editorType = 'html';
  editorShell.webShellWindow = window.frames[0];
  editorShell.contentWindow  = window.frames[0];
  editorShell.RegisterDocumentStateListener(listener);
  editorShell.contentsMIMEType = "text/html";
  editorShell.LoadUrl(url);
  
  return editorShell;
};

CommonEditor.prototype.cancel = function() {
    closeWindow();
};

CommonEditor.prototype.post = function() {
    var doc = document;
    // animate progress bar
    doc.getElementById("posting").mode = "undetermined";
    
    // disable the post and cancel buttons
    doc.getElementById("post").disabled = true;
    doc.getElementById("update").disabled = true;
    doc.getElementById("cancel").disabled = true;

    // updating
    if(this.updating){
        doc.getElementById("statusbar-text").setAttribute(
            "label",
            this.editorBundle.GetStringFromName("UPDATING_STATUS")
        );
    }
    // posting
    else{
        doc.getElementById("statusbar-text").setAttribute(
            "label",
            this.editorBundle.GetStringFromName("POSTING_STATUS")
        );
    }

    var xml = this.editor.getDocument();

    this.po.server = doc.getElementById("serverList").value;
    this.po.language = doc.getElementById("annotationLanguage").value;
    this.po.type = doc.getElementById("annotationType").value;
//    this.po.charset = defaultCharset;

    var result;
    try{
        result = this.po.post(xml);
    }
    catch(e){
        throw "Exception while posting the annotation: " + e;
    }
    if(!result){
        window.alert(
            this.editorBundle.formatStringFromName(
                (this.po.annotation)?("SERVER_ERROR1"):("SERVER_ERROR2"),
                [this.po.annotation.serverURI],
                1
            )
        );
    }
};

CommonEditor.prototype.spellCheck = function ()
{
    try {
        var args = [];
        args[0] = document.getElementById ("edit-frame").contentWindow.document.documentElement;
        window.openDialog("chrome://spellbound/content/SBSpellCheck.xul",
                "_blank", "chrome,close,titlebar,resizable,modal", false, true, true, args);

    } catch (e) {
        window.alert ("Unexpected error opening spelling checker: " + e);
    }
};

// from editor.js
CommonEditor.isSpellCheckerInstalled = function() 
{ 
  return "@mozilla.org/spellchecker;1" in Components.classes;
}; 

CommonEditor.prototype.hideItem = function(id)
{
  var item = document.getElementById(id);
  if (item) {
    item.setAttribute("hidden", "true");
  }
};

// from editorUtilities.js
CommonEditor.prototype.setElementEnabled = function (element, doEnable)
{
  if ( element ) {
    if ( doEnable ) {
      element.removeAttribute("disabled");
    }
    else {
      element.setAttribute("disabled", "true");
    }
  }
  else
  {
    dump("Element not found in SetElementEnabled\n");
  }
};


CommonEditor.getURLSpecFromFile = function(file) {
    
    if (Components.classes["@mozilla.org/network/io-service;1"] && Components.interfaces.nsIIOService) {
	
	var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
	var fileHandler = ioService.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);

	// new version
	if (fileHandler.getURLSpecFromFile) {
	    return fileHandler.getURLSpecFromFile(file);
	}
	// old version in ioService
	else {
	    return ioService.getURLSpecFromFile(file);
	}
    }
    // really old version
    else {
	var file_url = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIFileURL);
	file_url.file = file;
	return file_url.spec;
    }
};
